#*******************************************************************************
#*********************PROGRAMME HRS_angle***************************************
#*******************************************************************************
#   PySHS - An open source software about Second Harmonic Scattering 
#   developed at Institut Charles Gerhardt Montpellier - ENSCM
#   Dr Pierre-Marie GASSIN
#   Dr Gassin GASSIN
#    (june 2020)  
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#*******************************************************************************

#********************************************************************************
#
# Program for computing angle resolved SHS Intensity for molecules in solution
# with various polarization states.
#
#********************************************************************************


#*******************************************************************************
# IMPORT MODULES
#*******************************************************************************


import matplotlib.pyplot as plt
import numpy as np
import scipy as sp
import matplotlib as mpl
import math as mt
import sys
from scipy.integrate import tplquad
from mpl_toolkits.mplot3d import Axes3D

#input and output files
uv=sys.argv[1]
ooo=sys.argv[2]
ppp=sys.argv[2]+".pdf"

def Rphi(phi):
    aaa=np.array([[np.cos(phi),-np.sin(phi),0],[np.sin(phi),np.cos(phi),0],[0,0,1]])
    return aaa

def Rtheta(theta):
    bb=np.array([[1,0,0],[0,np.cos(theta),-np.sin(theta)],[0,np.sin(theta),np.cos(theta)]])
    return bb

def Rpsi(psi):
    cc=np.array([[np.cos(psi),-np.sin(psi),0],[np.sin(psi),np.cos(psi),0],[0,0,1]])
    return cc

def T(phi,theta,psi):
    zz=np.dot(np.linalg.inv(Rphi(phi)), np.dot(np.linalg.inv(Rtheta(theta)),np.linalg.inv(Rpsi(psi))))
    return zz


print("********************************************************")
print("****           PySHS: An open source software       ****")
print("****         about Second Harmonic Scattering       ****") 
print("**** Dr Pierre-Marie GASSIN and Dr Gaelle GASSIN    ****")          
print("****                  ICGM - ENSCM                  ****") 
print('****                   june 2020                    ****')
print("********************************************************")
pas2=int(input("number of points between 0 and 180°? "))

beta=np.zeros((3,3,3))
with open(uv,'r') as f:
    datacomments=f.readlines()[0]

kk=0
data=[0]*27
value=[0]*27
beta=np.zeros((3,3,3))
for i in range(3):
    for j in range(3):
        for k in range(3):
            with open(uv,'r') as f:
                data[kk]=f.readlines()[kk+1]
                value[kk] = data[kk].split()
                beta[i,j,k]=float(value[kk][1])
                kk=kk+1
                
print(f" hyperpolarisability tensor: \n {beta}")


def betalabijk(phi,theta,psi,i,j,k):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[0,0,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[0,0,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[0,0,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[0,1,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[0,1,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[0,1,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[0,2,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[0,2,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[0,2,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[1,0,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[1,0,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[1,0,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[1,1,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[1,1,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[1,1,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[1,2,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[1,2,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[1,2,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[2,0,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[2,0,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[2,0,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[2,1,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[2,1,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[2,1,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[2,2,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[2,2,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[2,2,2])
    return uuu



def betalabxxx_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,0,0,0)**2)*np.sin(theta)
    return uuu

def betalabxyy_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,0,1,1)**2)*np.sin(theta)
    return uuu

def betalabyxx(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,1,0,0)
    return uuu

def betalabyxx_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,1,0,0)**2)*np.sin(theta)
    return uuu

def betalabzxx(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,2,0,0)
    return uuu

def betalabzxx_yxx_sin(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,2,0,0)*betalabijk(phi,theta,psi,1,0,0)*np.sin(theta)
    return uuu

def betalabzxx_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,2,0,0)**2)*np.sin(theta)
    return uuu

def betalabzyy_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,2,1,1)**2)*np.sin(theta)
    return uuu

def betalabyyy_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,1,1,1)**2)*np.sin(theta)
    return uuu

def betalabzyy(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,2,1,1)
    return uuu

def betalabyyy(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,1,1,1)
    return uuu

def betalabzyy_yyy_sin(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,1,1,1)*betalabijk(phi,theta,psi,2,1,1)*np.sin(theta)
    return uuu


def betalabyxx(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,1,0,0)
    return uuu


def midpoint_triple1(g, a, b, c, d, e, f, nx, ny, nz):
    hx = (b - a)/float(nx)
    hy = (d - c)/float(ny)
    hz = (f - e)/float(nz)
    I = 0
    for i in range(nx):
        for j in range(ny):
            for k in range(nz):
                xi = a + hx/2 + i*hx
                yj = c + hy/2 + j*hy
                zk = e + hz/2 + k*hz
                I += hx*hy*hz*g(xi, yj, zk)
    return I

with open(ooo, "w") as f:    
    f.write(" ************************************************************** \n")
    f.write(" ************************program HRS_angle********************* \n")
    f.write(" ************************************************************** \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" *****Contributors : Pierre-Marie Gassin  &  Gaelle Gassin *****\n")
    f.write("  \n")
    f.write("  \n")
    f.write(" ************************INPUT********************************* \n")
    f.write("  \n")
    f.write(f" {datacomments} \n")
    f.write(f" hyperpolarisability tensor:   \n")
    f.write(f" {beta} \n")
    f.write("  \n")
    f.write(" **************************OUTPUT*******************************\n")
    f.write(f" grandtheta (°) IPSS-IH(0°) IPPP-IH(90°) ISSS-IV(0°) ISPP-IV(90°)  \n") 

grandthetat=np.zeros(2*(pas2)+1)
grandthetadegree=np.zeros(2*(pas2)+1)
ISSS=np.zeros(2*(pas2)+1)
IPPP=np.zeros(2*(pas2)+1)
ISPP=np.zeros(2*(pas2)+1)
IPSS=np.zeros(2*(pas2)+1)

print("********** Begin calculation **************")
for mmm in range(2*(pas2)+1):
    grandtheta=-np.pi+mmm*np.pi/pas2
    grandthetat[mmm]=grandtheta
    grandthetadegree[mmm]=grandtheta*360/(2*np.pi)
    print(f'grandtheta value: {grandthetadegree[mmm]:.5f}')
    ISSS[mmm]=midpoint_triple1(betalabxxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)
    print(f"IV(0) {ISSS[mmm]:.5f}")
    ISPP[mmm]=midpoint_triple1(betalabxyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)
    print(f"IV(90) {ISPP[mmm]:.5f}")    
    Iy0=midpoint_triple1(betalabyxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.cos(grandtheta)**4)+(midpoint_triple1(betalabzxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.cos(grandtheta)**2)*(np.sin(grandtheta)**2))-2*(midpoint_triple1(betalabzxx_yxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10))*(np.cos(grandtheta)**3)*(np.sin(grandtheta))
    Iz0=midpoint_triple1(betalabyxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.cos(grandtheta)**2)*(np.sin(grandtheta)**2)+(midpoint_triple1(betalabzxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.sin(grandtheta)**4))-2*(midpoint_triple1(betalabzxx_yxx_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10))*(np.cos(grandtheta))*(np.sin(grandtheta)**3)
    IPSS[mmm]=np.sqrt(Iy0**2+Iz0**2)
    print(f"IH(0) {IPSS[mmm]:.5f}")    
    Iy90=midpoint_triple1(betalabyyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.cos(grandtheta)**4)+(midpoint_triple1(betalabzyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.cos(grandtheta)**2)*(np.sin(grandtheta)**2))-2*(midpoint_triple1(betalabzyy_yyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10))*(np.cos(grandtheta)**3)*(np.sin(grandtheta))
    Iz90=midpoint_triple1(betalabyyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.cos(grandtheta)**2)*(np.sin(grandtheta)**2)+(midpoint_triple1(betalabzyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10)*(np.sin(grandtheta)**4))-2*(midpoint_triple1(betalabzyy_yyy_sin, 0, 2*np.pi, 0, np.pi,0, 2*np.pi, 10, 10, 10))*(np.cos(grandtheta))*(np.sin(grandtheta)**3)
    IPPP[mmm]=np.sqrt(Iy90**2+Iz90**2)
    print(f"IH(90) {IPPP[mmm]:.5f}")
    with open(ooo, "a") as f:   
        #f.write( " grandtheta %12.6f \n" % grandtheta)
        #f.write("IPPP  IPSS  ISSS  ISPP \n")
        #valeurs = "%12.6f"  "%12.6f"    "%12.6f"    "%12.6f  \n" % (IPPP,IPSS,ISSS,ISPP) 
        #f.write(valeurs)
        #valeurs2 = "%12.6f"  "%12.6f"  "%12.6f"    "%12.6f"     "%12.6f  \n" % (grandtheta,IPPP,IPSS,ISSS,ISPP) 
        f.write(f"{grandthetadegree[mmm]:.3f}      {IPSS[mmm]:.3f}      {IPPP[mmm]:.3f}       {ISSS[mmm]:.3f}      {ISPP[mmm]:.3f}   \n")  

print(f'calculation complete')
plt.plot(grandthetadegree,IPSS,label='IPSS-IH(0°)')
plt.plot(grandthetadegree,IPPP,label='IPPP-IH(90°)')
plt.plot(grandthetadegree,ISSS,label='ISSS-IV(0°)')
plt.plot(grandthetadegree,ISPP,label='ISPP-IV(90°)')
plt.legend(loc='upper left')
plt.xlabel('scattering angle[°]')
plt.ylabel('SHS Intensity [a.u]')
plt.savefig(ppp)
plt.show()

 

