#*******************************************************************************
#******************PROGRAMME HRS_polar_180**************************************
#*******************************************************************************
#   PySHS - An open source software about Second Harmonic Scattering software 
#   developed at Institut Charles Gerhardt Montpellier - ENSCM
#   Dr Pierre-Marie GASSIN
#   Dr Gassin GASSIN
#   (june 2020) 
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#*******************************************************************************

#********************************************************************************
#
# Program for computing polarization resolved SHS Intensity for molecules in solution 
#  at an angle of 180°
#********************************************************************************

#*******************************************************************************
# IMPORT MODULES
#*******************************************************************************
import matplotlib.pyplot as plt
import numpy as np
import sys

#input and output files
uv=sys.argv[1]
ooo=sys.argv[2]
ppp=sys.argv[2]+".pdf"

def Rphi(phi):
    aaa=np.array([[np.cos(phi),-np.sin(phi),0],[np.sin(phi),np.cos(phi),0],[0,0,1]])
    return aaa

def Rtheta(theta):
    bb=np.array([[1,0,0],[0,np.cos(theta),-np.sin(theta)],[0,np.sin(theta),np.cos(theta)]])
    return bb

def Rpsi(psi):
    cc=np.array([[np.cos(psi),-np.sin(psi),0],[np.sin(psi),np.cos(psi),0],[0,0,1]])
    return cc

def T(phi,theta,psi):
    zz=np.dot(np.linalg.inv(Rphi(phi)), np.dot(np.linalg.inv(Rtheta(theta)),np.linalg.inv(Rpsi(psi))))
    return zz

beta=np.zeros((3,3,3))

print("********************************************************")
print("****           PySHS: An open source software       ****")
print("****         about Second Harmonic Scattering       ****") 
print("**** Dr Pierre-Marie GASSIN and Dr Gaelle GASSIN    ****")          
print("****                  ICGM - ENSCM                  ****") 
print('****                   june 2020                    ****')
print("********************************************************")


with open(uv,'r') as f:
    datacomments=f.readlines()[0]

kk=0
data=[0]*27
value=[0]*27
beta=np.zeros((3,3,3))
for i in range(3):
    for j in range(3):
        for k in range(3):
            with open(uv,'r') as f:
                data[kk]=f.readlines()[kk+1]
                value[kk] = data[kk].split()
                beta[i,j,k]=float(value[kk][1])
                kk=kk+1
                
print(f" hyperpolarisability tensor: \n {beta}")

def betalabijk(phi,theta,psi,i,j,k):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[0,0,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[0,0,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[0,0,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[0,1,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[0,1,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[0,1,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[0,2,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[0,2,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[0,2,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[1,0,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[1,0,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[1,0,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[1,1,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[1,1,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[1,1,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[1,2,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[1,2,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[1,2,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[2,0,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[2,0,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[2,0,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[2,1,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[2,1,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[2,1,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[2,2,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[2,2,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[2,2,2])
    return uuu

def betalabxxx_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,0,0,0)**2)*np.sin(theta)
    return uuu

def betalabxyy_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,0,1,1)**2)*np.sin(theta)
    return uuu

def betalabyxx_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,1,0,0)**2)*np.sin(theta)
    return uuu

def betalabyyy_sin(phi,theta,psi):
    uuu=0
    uuu=(betalabijk(phi,theta,psi,1,1,1)**2)*np.sin(theta)
    return uuu

def beta_bv(phi,theta,psi):
    uuu=0
    uuu=(4*(betalabijk(phi,theta,psi,0,0,1)**2)+2*betalabijk(phi,theta,psi,0,0,0)*betalabijk(phi,theta,psi,0,1,1))*np.sin(theta)
    return uuu

def beta_bh(phi,theta,psi):
    uuu=0
    uuu=(4*(betalabijk(phi,theta,psi,1,0,1)**2)+2*betalabijk(phi,theta,psi,1,0,0)*betalabijk(phi,theta,psi,1,1,1))*np.sin(theta)
    return uuu

def midpoint_triple1(g, a, b, c, d, e, f, nx, ny, nz):
    hx = (b - a)/float(nx)
    hy = (d - c)/float(ny)
    hz = (f - e)/float(nz)
    I = 0
    for i in range(nx):
        for j in range(ny):
            for k in range(nz):
                xi = a + hx/2 + i*hx
                yj = c + hy/2 + j*hy
                zk = e + hz/2 + k*hz
                I += hx*hy*hz*g(xi, yj, zk)
    return I


av=0
bv=0
bv1=0
bv2=0
cv=0
ah=0
bh=0
bh1=0
bh2=0
ch=0

hx = (2*np.pi)/float(16)
hy = (np.pi)/float(16)
hz = (2*np.pi)/float(16)
I = 0

print("********** Begin calculation **************")
for i in range(16):
    fgf=float(i/16)*100
    print(f"calculation in progress: {fgf:.0f} %")    
    for j in range(16):
        for k in range(16):
            xi = 0 + hx/2 + i*hx
            yj = 0 + hy/2 + j*hy
            zk = 0 + hz/2 + k*hz
            av=av+hx*hy*hz*betalabxxx_sin(xi, yj, zk)
            bv=bv+hx*hy*hz*beta_bv(xi, yj, zk)
            cv=cv+hx*hy*hz*betalabxyy_sin(xi, yj, zk)
            ah=ah+hx*hy*hz*betalabyxx_sin(xi, yj, zk)
            bh=bh+hx*hy*hz*beta_bh(xi, yj, zk)
            ch=ch+hx*hy*hz*betalabyyy_sin(xi, yj, zk)
            
print("**********calculation complete ************")   
print(f"av = {av:.5f}")
print(f"bv = {bv:.5f}")
print(f"cv = {cv:.5f}")
print(f"ah = {ah:.5f}")
print(f"bh = {bh:.5f}")
print(f"ch =  {ch:.5f}")    

I4v=(av-bv+cv)/((3*av+bv+3*cv))
I2v=(4)*(av-cv)/(3*av+bv+3*cv)
I4h=(ah-bh+ch)/((3*ah+bh+3*ch))
I2h=(4)*(ah-ch)/(3*ah+bh+3*ch)
print(f"I2v = {I2v:.5f}")
print(f"I4v = {I4v:.5f}")
print(f"I2h = {I2h:.5f}")
print(f"I4h = {I4h:.5f}")


with open(ooo, "w") as f:    
    f.write(" **************************************************** \n")
    f.write(" ******************program HRS_polar_180************* \n")
    f.write(" **************************************************** \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" Contributors : Pierre-Marie Gassin  &  Gaelle Gassin \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" ********************INPUT*************************** \n")
    f.write("  \n")
    f.write(f" {datacomments} \n")
    f.write(f" hyperpolarisability tensor:   \n")
    f.write(f" {beta} \n")
    f.write("  \n")
    f.write(" ****************OUTPUT*******************************\n")
    f.write("  \n")
    f.write("  \n")
    f.write("av %12.6f\n" % av)
    f.write("bv %12.6f\n" % bv)
 #   f.write("bv1 %12.6f\n" % bv1)
 #   f.write("bv2 %12.6f\n" % bv2)
    f.write("cv %12.6f\n" % cv)
    f.write("ah %12.6f\n" % ah)
    f.write("bh %12.6f\n" % bh)
 #   f.write("bh1 %12.6f\n" % bh1)
 #   f.write("bh2 %12.6f\n" % bh2)
    f.write("ch %12.6f\n" % ch)
    f.write("  \n")
    f.write("*************************************************\n")
    f.write("  \n")
    f.write("I2v %12.6f\n" % I2v)
    f.write("I4v %12.6f\n" % I4v)
    f.write("I2h %12.6f\n" % I2h)   
    f.write("I4h %12.6f\n" % I4h)
    f.write("  \n")
    f.write("*************************************************\n")
    f.write("Have a nice day and enjoy the life!")


gammaa=np.linspace(0,2*np.pi,100)
plt.plot(gammaa*360/(2*np.pi),(av)*((np.cos(gammaa))**4)+(bv)*((np.cos(gammaa))**2)*((np.sin(gammaa))**2)+(cv)*((np.sin(gammaa))**4),label='polarV_out')#+(bv1)*((np.cos(gammaa))**3)*((np.sin(gammaa)))+(bv2)*((np.cos(gammaa)))*((np.sin(gammaa))**3)+(cv)*((np.sin(gammaa))**4))
plt.plot(gammaa*360/(2*np.pi),(ah)*((np.cos(gammaa))**4)+(bh)*((np.cos(gammaa))**2)*((np.sin(gammaa))**2)+(ch)*((np.sin(gammaa))**4),label='polarH_out')#+(bh1)*((np.cos(gammaa))**3)*((np.sin(gammaa)))+(bh2)*((np.cos(gammaa)))*((np.sin(gammaa))**3)+(ch)*((np.sin(gammaa))**4))
plt.legend(loc='upper left')
plt.ylabel("SHS Intensity in the forward direction (180°)")
plt.xlabel("polarization angle [degree]")
plt.savefig(ppp)
plt.show()
