
#*******************************************************************************
#******************PROGRAM SHS_angle********************************************
#*******************************************************************************
#   PySHS - An open source software about Second Harmonic Scattering 
#   developed at Institut Charles Gerhardt Montpellier - ENSCM
#   Dr Pierre-Marie GASSIN
#   Dr Gassin GASSIN
#    (june 2020)  
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#*******************************************************************************

#********************************************************************************
#
# Program for computing angle resolved SHS Intensity for supramolecular
# structure in solution
#  
#********************************************************************************

#*******************************************************************************
# IMPORT MODULES
#*******************************************************************************


import matplotlib.pyplot as plt
import numpy as np
import matplotlib as mpl
import sys
from scipy.integrate import tplquad
from mpl_toolkits.mplot3d import Axes3D

#input and output files

ukv=sys.argv[1]
uv=sys.argv[2]
uu=sys.argv[3]
uuu=sys.argv[3]+".pdf"
vvv=sys.argv[3]+".png"

print("********************************************************")
print("****           PySHS: An open source software       ****")
print("****         about Second Harmonic Scattering       ****") 
print("**** Dr Pierre-Marie GASSIN and Dr Gaelle GASSIN    ****")          
print("****                  ICGM - ENSCM                  ****") 
print('****                   june 2020                    ****')
print("********************************************************")

n=float(input("refractive index of the medium?"))
longueuronde=float(input("wavelength of the incident laser beam (expressed in nm)?"))
nombre=int(input("number of dipoles (ie number of lines in the input file)?"))
pas2=int(input("number of points between 0 and 180° wanted to plot the angle resolved SHS?"))

petitk=2*np.pi*n/longueuronde
grandk=4*np.pi*n/longueuronde
pas=8

def Rphi(phi):
    aaa=np.array([[np.cos(phi),-np.sin(phi),0],[np.sin(phi),np.cos(phi),0],[0,0,1]])
    return aaa

def Rtheta(theta):
    bb=np.array([[1,0,0],[0,np.cos(theta),-np.sin(theta)],[0,np.sin(theta),np.cos(theta)]])
    return bb

def Rpsi(psi):
    cc=np.array([[np.cos(psi),-np.sin(psi),0],[np.sin(psi),np.cos(psi),0],[0,0,1]])
    return cc


def T(phi,theta,psi):
    zz=np.dot(np.linalg.inv(Rphi(phi)), np.dot(np.linalg.inv(Rtheta(theta)),np.linalg.inv(Rpsi(psi))))
    return zz

def betalabijk(phi,theta,psi,i,j,k,number):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betamesoxxx[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betamesoxxy[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betamesoxxz[number])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betamesoxyx[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betamesoxyy[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betamesoxyz[number])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betamesoxzx[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betamesoxzy[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betamesoxzz[number])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betamesoyxx[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betamesoyxy[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betamesoyxz[number])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betamesoyyx[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betamesoyyy[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betamesoyyz[number])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betamesoyzx[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betamesoyzy[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betamesoyzz[number])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betamesozxx[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betamesozxy[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betamesozxz[number])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betamesozyx[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betamesozyy[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betamesozyz[number])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betamesozzx[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betamesozzy[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betamesozzz[number])
    return uuu

def xyzlabijk(phi,theta,psi,i,number):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*x[number])+(T(phi,theta,psi)[i,1]*y[number])+(T(phi,theta,psi)[i,2]*z[number])
    return uuu

def xlabo(phi,theta,psi,number):
    uuu=0
    uuu=xyzlabijk(phi,theta,psi,0,number)
    return uuu

def ylabo(phi,theta,psi,number):
    uuu=0
    uuu=xyzlabijk(phi,theta,psi,1,number)
    return uuu

def zlabo(phi,theta,psi,number):
    uuu=0
    uuu=xyzlabijk(phi,theta,psi,2,number)
    return uuu

def betaeffX(phi,theta,psi,gammaa):
    uuu=0
    uuu=(betalabxxx(phi,theta,psi)*(np.cos(gammaa))**2)+2*(betalabxxy(phi,theta,psi)*(np.cos(gammaa))*(np.sin(gammaa)))+(betalabxyy(phi,theta,psi)*(np.sin(gammaa))**2)
    return uuu

def betaeffZ(phi,theta,psi,gammaa):
    uuu=0
    uuu=(betalabzxx(phi,theta,psi)*(np.cos(gammaa))**2)+2*(betalabzxy(phi,theta,psi)*(np.cos(gammaa))*(np.sin(gammaa)))+(betalabzyy(phi,theta,psi)*(np.sin(gammaa))**2)
    return uuu

def betamesoijk(phi,theta,psi,i,j,k):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[0,0,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[0,0,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[0,0,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[0,1,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[0,1,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[0,1,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[0,2,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[0,2,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[0,2,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[1,0,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[1,0,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[1,0,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[1,1,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[1,1,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[1,1,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[1,2,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[1,2,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[1,2,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[2,0,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[2,0,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[2,0,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[2,1,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[2,1,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[2,1,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[2,2,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[2,2,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[2,2,2])
    return uuu

def betalabzyy_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,2,1,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabyyy_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,1,1,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabxxx_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,0,0,0,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabzxx_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,2,0,0,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabxyy_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,0,1,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabxxy_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,0,0,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabzxy_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,2,0,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

def betalabyxx_retard(phi,theta,psi,ii,grandtheta):
    uuu=complex(0,0)
    uuu=betalabijk(phi,theta,psi,1,0,0,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*(2*petitk-grandk*np.cos(grandtheta))-ylabo(phi,theta,psi,ii)*grandk*np.sin(grandtheta))))
    return uuu

beta=np.zeros((3,3,3))
with open(ukv,'r') as f:
    datacomments=f.readlines()[0]


kk=0
data1=[0]*27
value1=[0]*27
beta=np.zeros((3,3,3))
for i in range(3):
    for j in range(3):
        for k in range(3):
            with open(ukv,'r') as f:
                data1[kk]=f.readlines()[kk+1]
                value1[kk] = data1[kk].split()
                beta[i,j,k]=float(value1[kk][1])
                kk=kk+1
                
print(f" input hyperpolarisability tensor: \n {beta}")

nn=np.zeros(nombre)
phi=np.zeros(nombre)
theta=np.zeros(nombre)
psi=np.zeros(nombre)
x=np.zeros(nombre)
y=np.zeros(nombre)
z=np.zeros(nombre)

betamesoxxx=np.zeros(nombre)
betamesoxxy=np.zeros(nombre)
betamesoxxz=np.zeros(nombre)
betamesoxyx=np.zeros(nombre)
betamesoxyy=np.zeros(nombre)
betamesoxyz=np.zeros(nombre)
betamesoxzx=np.zeros(nombre)
betamesoxzy=np.zeros(nombre)
betamesoxzz=np.zeros(nombre)

betamesoyxx=np.zeros(nombre)
betamesoyxy=np.zeros(nombre)
betamesoyxz=np.zeros(nombre)
betamesoyyx=np.zeros(nombre)
betamesoyyy=np.zeros(nombre)
betamesoyyz=np.zeros(nombre)
betamesoyzx=np.zeros(nombre)
betamesoyzy=np.zeros(nombre)
betamesoyzz=np.zeros(nombre)

betamesozxx=np.zeros(nombre)
betamesozxy=np.zeros(nombre)
betamesozxz=np.zeros(nombre)
betamesozyx=np.zeros(nombre)
betamesozyy=np.zeros(nombre)
betamesozyz=np.zeros(nombre)
betamesozzx=np.zeros(nombre)
betamesozzy=np.zeros(nombre)
betamesozzz=np.zeros(nombre)

def midpoint_triple1(g, a, b, c, d, e, f, nx, ny, nz,ii):
    hx = (b - a)/float(nx)
    hy = (d - c)/float(ny)
    hz = (f - e)/float(nz)
    I = 0
    for i in range(nx):
        for j in range(ny):
            for k in range(nz):
                xi = a + hx/2 + i*hx
                yj = c + hy/2 + j*hy
                zk = e + hz/2 + k*hz
                I += hx*hy*hz*g(xi, yj, zk,ii)
    return I

av=complex(0,0)
cv=complex(0,0)
ah=complex(0,0)
ch=complex(0,0)
bv=complex(0,0)
bv1=complex(0,0)
bv2=complex(0,0)
bh=complex(0,0)
bh1=complex(0,0)
bh2=complex(0,0)
data=[0]*nombre
value=[0]*nombre
for i in range(nombre):
    with open(uv,'r') as f:
        data[i]=f.readlines()[i]
        value[i] = data[i].split()
        phi[i]=float(value[i][0])
        theta[i]=float(value[i][1])
        psi[i]=float(value[i][2])
        x[i]=float(value[i][3])
        y[i]=float(value[i][4])
        z[i]=float(value[i][5])
    betamesoxxx[i]=betamesoijk(phi[i],theta[i],psi[i],0,0,0)
    betamesoxxy[i]=betamesoijk(phi[i],theta[i],psi[i],0,0,1)
    betamesoxxz[i]=betamesoijk(phi[i],theta[i],psi[i],0,0,2)
    betamesoxyx[i]=betamesoijk(phi[i],theta[i],psi[i],0,1,0)
    betamesoxyy[i]=betamesoijk(phi[i],theta[i],psi[i],0,1,1)
    betamesoxyz[i]=betamesoijk(phi[i],theta[i],psi[i],0,1,2)
    betamesoxzx[i]=betamesoijk(phi[i],theta[i],psi[i],0,2,0)
    betamesoxzy[i]=betamesoijk(phi[i],theta[i],psi[i],0,2,1)
    betamesoxzz[i]=betamesoijk(phi[i],theta[i],psi[i],0,2,2)
    betamesoyxx[i]=betamesoijk(phi[i],theta[i],psi[i],1,0,0)
    betamesoyxy[i]=betamesoijk(phi[i],theta[i],psi[i],1,0,1)
    betamesoyxz[i]=betamesoijk(phi[i],theta[i],psi[i],1,0,2)
    betamesoyyx[i]=betamesoijk(phi[i],theta[i],psi[i],1,1,0)
    betamesoyyy[i]=betamesoijk(phi[i],theta[i],psi[i],1,1,1)
    betamesoyyz[i]=betamesoijk(phi[i],theta[i],psi[i],1,1,2)
    betamesoyzx[i]=betamesoijk(phi[i],theta[i],psi[i],1,2,0)
    betamesoyzy[i]=betamesoijk(phi[i],theta[i],psi[i],1,2,1)
    betamesoyzz[i]=betamesoijk(phi[i],theta[i],psi[i],1,2,2)
    betamesozxx[i]=betamesoijk(phi[i],theta[i],psi[i],2,0,0)
    betamesozxy[i]=betamesoijk(phi[i],theta[i],psi[i],2,0,1)
    betamesozxz[i]=betamesoijk(phi[i],theta[i],psi[i],2,0,2)
    betamesozyx[i]=betamesoijk(phi[i],theta[i],psi[i],2,1,0)
    betamesozyy[i]=betamesoijk(phi[i],theta[i],psi[i],2,1,1)
    betamesozyz[i]=betamesoijk(phi[i],theta[i],psi[i],2,1,2)
    betamesozzx[i]=betamesoijk(phi[i],theta[i],psi[i],2,2,0)
    betamesozzy[i]=betamesoijk(phi[i],theta[i],psi[i],2,2,1)
    betamesozzz[i]=betamesoijk(phi[i],theta[i],psi[i],2,2,2)

with open(uu, "w") as f:
    f.write(" ************************************************************* \n")
    f.write(" *******************program_SHS_angle************************* \n")
    f.write(" ************************************************************* \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" ********Contributors : Pierre-Marie  &  Gaelle Gassin ********\n")
    f.write("  \n")
    f.write("  \n")
    f.write(" ********************INPUT************************************ \n")
    f.write("  \n")
    f.write(f" refractive index of the medium: {n}")
    f.write(f" wavelength of the incident laser beam (expressed in nm): {longueuronde}")
    f.write(f" number of dipoles: {nombre}")
    f.write(f" {datacomments} \n")
    f.write(f" hyperpolarisability tensor:   \n")
    f.write(f" {beta} \n")
    f.write("  \n")
    f.write(" ****************OUTPUT****************************************\n")
    f.write(f" grandtheta  IPSS-IH(0°)   IPPP-IH(90°)   ISSS-IV(0°)   ISPP-IV(90°) \n") 

grandthetat=np.zeros(2*pas2+1)
grandthetadegree=np.zeros(2*pas2+1)
ISSS=np.zeros(2*pas2+1)
IPPP=np.zeros(2*pas2+1)
ISPP=np.zeros(2*pas2+1)
IPSS=np.zeros(2*pas2+1)

hx = (2*np.pi)/float(pas)
hy = (np.pi)/float(pas)
hz = (2*np.pi)/float(pas)
nx=pas
ny=pas
nz=pas

print("********** Begin calculation **************")

for mmm in range(2*(pas2)+1):
    grandtheta=-np.pi+mmm*np.pi/pas2
    grandthetat[mmm]=grandtheta
    grandthetadegree[mmm]=grandtheta*360/(2*np.pi)
    print(f'angle value: {grandthetadegree[mmm]:.5f}')
    beta_y_moy_0=0
    beta_z_moy_0=0
    beta_y_moy_90=0
    beta_z_moy_90=0
    beta_xxx_moy=0
    beta_xyy_moy=0
    for i in range(nx):
        fgf=float(i/nx)*100
        print(f"calculation in progress: {fgf:.0f} %")
        for j in range(ny):
            for kk in range(nz):
                xi = 0 + hx/2 + i*hx
                yj = 0 + hy/2 + j*hy
                zk = 0 + hz/2 + kk*hz
                betalabozyyint=complex(0,0)
                betalabozxxint=complex(0,0)                 
                betalaboyxxint=complex(0,0)
                betalaboyyyint=complex(0,0)
                betalaboxyyint=complex(0,0)
                betalaboxxxint=complex(0,0)
                for ii in range(nombre):
                    betalabozxxint=betalabozxxint+betalabzxx_retard(xi, yj, zk,ii,grandtheta)
                    betalaboyxxint=betalaboyxxint+betalabyxx_retard(xi, yj, zk,ii,grandtheta)
                    betalabozyyint=betalabozyyint+betalabzxx_retard(xi, yj, zk,ii,grandtheta)
                    betalaboyyyint=betalaboyyyint+betalabyyy_retard(xi, yj, zk,ii,grandtheta)
                    betalaboxyyint=betalaboxyyint+betalabxyy_retard(xi, yj, zk,ii,grandtheta)
                    betalaboxxxint=betalaboxxxint+betalabxxx_retard(xi, yj, zk,ii,grandtheta)            
                beta_xxx_moy=beta_xxx_moy+hx*hy*hz*(abs(betalaboxxxint)**2)*np.sin(yj)
                beta_xyy_moy=beta_xyy_moy+hx*hy*hz*(abs(betalaboxyyint)**2)*np.sin(yj)
                beta_y_moy_0=beta_y_moy_0+hx*hy*hz*(betalaboyxxint*(np.cos(grandtheta)**2)-betalabozxxint*np.cos(grandtheta)*np.sin(grandtheta))*np.conjugate(betalaboyxxint*(np.cos(grandtheta)**2)-betalabozxxint*np.cos(grandtheta)*np.sin(grandtheta))
                beta_z_moy_0=beta_z_moy_0+hx*hy*hz*(-betalaboyxxint*(np.cos(grandtheta)*np.sin(grandtheta))+betalabozxxint*(np.sin(grandtheta)**2))*np.conjugate(-betalaboyxxint*(np.cos(grandtheta)*np.sin(grandtheta))+betalabozxxint*(np.sin(grandtheta)**2))
                beta_y_moy_90=beta_y_moy_90+hx*hy*hz*(betalaboyyyint*(np.cos(grandtheta)**2)-betalabozyyint*np.cos(grandtheta)*np.sin(grandtheta))*np.conjugate(betalaboyyyint*(np.cos(grandtheta)**2)-betalabozyyint*np.cos(grandtheta)*np.sin(grandtheta))
                beta_z_moy_90=beta_z_moy_90+hx*hy*hz*(-betalaboyyyint*(np.cos(grandtheta)*np.sin(grandtheta))+betalabozyyint*(np.sin(grandtheta)**2))*np.conjugate(-betalaboyyyint*(np.cos(grandtheta)*np.sin(grandtheta))+betalabozyyint*(np.sin(grandtheta)**2))
    IPSS[mmm]=(np.sqrt(beta_y_moy_0**2+beta_z_moy_0**2)).real
    IPPP[mmm]=(np.sqrt(beta_y_moy_90**2+beta_z_moy_90**2)).real
    ISSS[mmm]=beta_xxx_moy
    ISPP[mmm]=beta_xyy_moy
    print(f"IPSS={IPSS[mmm]:.3f}    IPPP={IPPP[mmm]:.3f}      ISSS={ISSS[mmm]:.3f}    ISPP={ISPP[mmm]:.3f}")
    with open(uu, "a") as f:   
        f.write(f"{grandthetadegree[mmm]:.3f}     {IPSS[mmm]:.3f}     {IPPP[mmm]:.3f}      {ISSS[mmm]:.3f}    {ISPP[mmm]:.3f}  \n")   

print(f'calculation finished - Have a nice day!')
plt.plot(grandthetadegree,IPSS,label='IPSS=IH(0°)')
plt.plot(grandthetadegree,IPPP,label='IPPP=IH(90°)')
plt.plot(grandthetadegree,ISSS,label='ISSS=IV(0°)')
plt.plot(grandthetadegree,ISPP,label='ISPP=IV(90°)')
plt.legend(loc='upper left')
plt.xlabel('scattering angle[°]')
plt.ylabel('SHS Intensity [a.u]')
plt.savefig(uuu)
plt.show()
        
fig = plt.figure()
ax = fig.gca(projection='3d')
ax.quiver(x,y,z,np.sin(phi)*np.sin(theta),np.cos(phi)*np.sin(theta),np.cos(theta), normalize=True)
plt.savefig(vvv)
plt.show()


