#*******************************************************************************
#******************PROGRAM SHS_polar_90******************************************
#*******************************************************************************
#   PySHS - An open source software about Second Harmonic Scattering 
#   developed at Institut Charles Gerhardt Montpellier - ENSCM
#   Dr Pierre-Marie GASSIN
#   Dr Gassin GASSIN
#    (june 2020)  
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#*******************************************************************************

#********************************************************************************
#
# Program for computing polarization resolved SHS Intensity for supramolecular
# structure in solution in the right angle direction
#  
#********************************************************************************

#*******************************************************************************
# IMPORT MODULES
#*******************************************************************************

import matplotlib.pyplot as plt
import numpy as np
import matplotlib as mpl
import sys
from scipy.integrate import tplquad
from mpl_toolkits.mplot3d import Axes3D


#input and output files
ukv=sys.argv[1]
uv=sys.argv[2]
uu=sys.argv[3]
uuu=sys.argv[3]+".pdf"
vvv=sys.argv[3]+"_configuration"+".pdf"


print("********************************************************")
print("****           PySHS: An open source software       ****")
print("****         about Second Harmonic Scattering       ****") 
print("**** Dr Pierre-Marie GASSIN and Dr Gaelle GASSIN    ****")          
print("****                  ICGM - ENSCM                  ****") 
print('****                   june 2020                    ****')
print("********************************************************")

n=float(input("refractive index of the medium?"))
longueuronde=float(input("wavelength of the incident laser beam (expressed in nm)?"))
nombre=int(input("number of dipoles (ie number of lines in the input file)?"))
grandk=4*np.pi*n/longueuronde
petitk=2*np.pi*n/longueuronde


def Rphi(phi):
    aaa=np.array([[np.cos(phi),-np.sin(phi),0],[np.sin(phi),np.cos(phi),0],[0,0,1]])
    return aaa

def Rtheta(theta):
    bb=np.array([[1,0,0],[0,np.cos(theta),-np.sin(theta)],[0,np.sin(theta),np.cos(theta)]])
    return bb

def Rpsi(psi):
    cc=np.array([[np.cos(psi),-np.sin(psi),0],[np.sin(psi),np.cos(psi),0],[0,0,1]])
    return cc


def T(phi,theta,psi):
    zz=np.dot(np.linalg.inv(Rphi(phi)), np.dot(np.linalg.inv(Rtheta(theta)),np.linalg.inv(Rpsi(psi))))
    return zz

def betalabijk(phi,theta,psi,i,j,k,number):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betamesoxxx[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betamesoxxy[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betamesoxxz[number])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betamesoxyx[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betamesoxyy[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betamesoxyz[number])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betamesoxzx[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betamesoxzy[number])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betamesoxzz[number])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betamesoyxx[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betamesoyxy[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betamesoyxz[number])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betamesoyyx[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betamesoyyy[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betamesoyyz[number])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betamesoyzx[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betamesoyzy[number])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betamesoyzz[number])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betamesozxx[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betamesozxy[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betamesozxz[number])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betamesozyx[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betamesozyy[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betamesozyz[number])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betamesozzx[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betamesozzy[number])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betamesozzz[number])
    return uuu

def xyzlabijk(phi,theta,psi,i,number):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*x[number])+(T(phi,theta,psi)[i,1]*y[number])+(T(phi,theta,psi)[i,2]*z[number])
    return uuu

def xlabo(phi,theta,psi,number):
    uuu=0
    uuu=xyzlabijk(phi,theta,psi,0,number)
    return uuu

def ylabo(phi,theta,psi,number):
    uuu=0
    uuu=xyzlabijk(phi,theta,psi,1,number)
    return uuu

def zlabo(phi,theta,psi,number):
    uuu=0
    uuu=xyzlabijk(phi,theta,psi,2,number)
    return uuu

def betalabzyy_retard(phi,theta,psi,ii):
    uuuvv=(0)
    uuuvv=betalabijk(phi,theta,psi,2,1,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*2*petitk-ylabo(phi,theta,psi,ii)*grandk)))
    return uuuvv

def betalabxxx_retard(phi,theta,psi,ii):
    uuuvv=(0)
    uuuvv=betalabijk(phi,theta,psi,0,0,0,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*2*petitk-ylabo(phi,theta,psi,ii)*grandk)))
    return uuuvv

def betalabzxx_retard(phi,theta,psi,ii):
    uuuvv=(0)
    uuuvv=betalabijk(phi,theta,psi,2,0,0,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*2*petitk-ylabo(phi,theta,psi,ii)*grandk)))
    return uuuvv

def betalabxyy_retard(phi,theta,psi,ii):
    uuuvv=(0)
    uuuvv=betalabijk(phi,theta,psi,0,1,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*2*petitk-ylabo(phi,theta,psi,ii)*grandk)))
    return uuuvv

def betalabxxy_retard(phi,theta,psi,ii):
    uuuvv=(0)
    uuuvv=betalabijk(phi,theta,psi,0,0,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*2*petitk-ylabo(phi,theta,psi,ii)*grandk)))
    return uuuvv

def betalabzxy_retard(phi,theta,psi,ii):
    uuuvv=(0)
    uuuvv=betalabijk(phi,theta,psi,2,0,1,ii)*(np.exp(complex(0,zlabo(phi,theta,psi,ii)*2*petitk-ylabo(phi,theta,psi,ii)*grandk)))
    return uuuvv


def betamesoijk(phi,theta,psi,i,j,k):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[0,0,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[0,0,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[0,0,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[0,1,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[0,1,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[0,1,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[0,2,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[0,2,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[0,2,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[1,0,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[1,0,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[1,0,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[1,1,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[1,1,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[1,1,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[1,2,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[1,2,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[1,2,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[2,0,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[2,0,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[2,0,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[2,1,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[2,1,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[2,1,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[2,2,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[2,2,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[2,2,2])
    return uuu


beta=np.zeros((3,3,3))
with open(ukv,'r') as f:
    datacomments=f.readlines()[0]

kk=0
data1=[0]*27
value1=[0]*27
beta=np.zeros((3,3,3))
for i in range(3):
    for j in range(3):
        for k in range(3):
            with open(ukv,'r') as f:
                data1[kk]=f.readlines()[kk+1]
                value1[kk] = data1[kk].split()
                beta[i,j,k]=float(value1[kk][1])
                kk=kk+1

print(f" input hyperpolarisability tensor: \n {beta}")


nn=np.zeros(nombre)
phi=np.zeros(nombre)
theta=np.zeros(nombre)
psi=np.zeros(nombre)
x=np.zeros(nombre)
y=np.zeros(nombre)
z=np.zeros(nombre)

betamesoxxx=np.zeros(nombre)
betamesoxxy=np.zeros(nombre)
betamesoxxz=np.zeros(nombre)
betamesoxyx=np.zeros(nombre)
betamesoxyy=np.zeros(nombre)
betamesoxyz=np.zeros(nombre)
betamesoxzx=np.zeros(nombre)
betamesoxzy=np.zeros(nombre)
betamesoxzz=np.zeros(nombre)

betamesoyxx=np.zeros(nombre)
betamesoyxy=np.zeros(nombre)
betamesoyxz=np.zeros(nombre)
betamesoyyx=np.zeros(nombre)
betamesoyyy=np.zeros(nombre)
betamesoyyz=np.zeros(nombre)
betamesoyzx=np.zeros(nombre)
betamesoyzy=np.zeros(nombre)
betamesoyzz=np.zeros(nombre)

betamesozxx=np.zeros(nombre)
betamesozxy=np.zeros(nombre)
betamesozxz=np.zeros(nombre)
betamesozyx=np.zeros(nombre)
betamesozyy=np.zeros(nombre)
betamesozyz=np.zeros(nombre)
betamesozzx=np.zeros(nombre)
betamesozzy=np.zeros(nombre)
betamesozzz=np.zeros(nombre)


av=0
cv=0
ah=0
ch=0
bv=0
bv1=0
bv2=0
bh=0
bh1=0
bh2=0
data=[0]*nombre
value=[0]*nombre
for i in range(nombre):
    with open(uv,'r') as f:
        data[i]=f.readlines()[i]
        value[i] = data[i].split()
        phi[i]=float(value[i][0])
        theta[i]=float(value[i][1])
        psi[i]=float(value[i][2])
        x[i]=float(value[i][3])
        y[i]=float(value[i][4])
        z[i]=float(value[i][5])
    betamesoxxx[i]=betamesoijk(phi[i],theta[i],psi[i],0,0,0)
    betamesoxxy[i]=betamesoijk(phi[i],theta[i],psi[i],0,0,1)
    betamesoxxz[i]=betamesoijk(phi[i],theta[i],psi[i],0,0,2)
    betamesoxyx[i]=betamesoijk(phi[i],theta[i],psi[i],0,1,0)
    betamesoxyy[i]=betamesoijk(phi[i],theta[i],psi[i],0,1,1)
    betamesoxyz[i]=betamesoijk(phi[i],theta[i],psi[i],0,1,2)
    betamesoxzx[i]=betamesoijk(phi[i],theta[i],psi[i],0,2,0)
    betamesoxzy[i]=betamesoijk(phi[i],theta[i],psi[i],0,2,1)
    betamesoxzz[i]=betamesoijk(phi[i],theta[i],psi[i],0,2,2)
    betamesoyxx[i]=betamesoijk(phi[i],theta[i],psi[i],1,0,0)
    betamesoyxy[i]=betamesoijk(phi[i],theta[i],psi[i],1,0,1)
    betamesoyxz[i]=betamesoijk(phi[i],theta[i],psi[i],1,0,2)
    betamesoyyx[i]=betamesoijk(phi[i],theta[i],psi[i],1,1,0)
    betamesoyyy[i]=betamesoijk(phi[i],theta[i],psi[i],1,1,1)
    betamesoyyz[i]=betamesoijk(phi[i],theta[i],psi[i],1,1,2)
    betamesoyzx[i]=betamesoijk(phi[i],theta[i],psi[i],1,2,0)
    betamesoyzy[i]=betamesoijk(phi[i],theta[i],psi[i],1,2,1)
    betamesoyzz[i]=betamesoijk(phi[i],theta[i],psi[i],1,2,2)
    betamesozxx[i]=betamesoijk(phi[i],theta[i],psi[i],2,0,0)
    betamesozxy[i]=betamesoijk(phi[i],theta[i],psi[i],2,0,1)
    betamesozxz[i]=betamesoijk(phi[i],theta[i],psi[i],2,0,2)
    betamesozyx[i]=betamesoijk(phi[i],theta[i],psi[i],2,1,0)
    betamesozyy[i]=betamesoijk(phi[i],theta[i],psi[i],2,1,1)
    betamesozyz[i]=betamesoijk(phi[i],theta[i],psi[i],2,1,2)
    betamesozzx[i]=betamesoijk(phi[i],theta[i],psi[i],2,2,0)
    betamesozzy[i]=betamesoijk(phi[i],theta[i],psi[i],2,2,1)
    betamesozzz[i]=betamesoijk(phi[i],theta[i],psi[i],2,2,2)

xmax=np.amax(np.absolute(x))
ymax=np.amax(np.absolute(y))
zmax=np.amax(np.absolute(z))
maxtotal=(xmax+ymax+zmax)
pas=10
nx=2*pas
ny=pas
nz=2*pas
hx = (2*np.pi)/float(nx)
hy = (np.pi)/float(ny)
hz = (2*np.pi)/float(nz)
print("********** Begin calculation **************")
for i in range(nx):
    fgf=float(i/nx)*100
    print(f"calculation in progress: {fgf:.0f} %")
    for j in range(ny):
        for kkk in range(nz):
            xi = 0 + hx/2 + i*hx
            yj = 0 + hy/2 + j*hy
            zk = 0 + hz/2 + kkk*hz
            betalabozyyint=complex(0,0)
            betalaboxxxint=complex(0,0)
            betalabozxxint=complex(0,0)
            betalaboxyyint=complex(0,0)
            betalaboxxyint=complex(0,0)
            betalabozxyint=complex(0,0)
            for ii in range(nombre):
                betalabozyyint=betalabozyyint+betalabzyy_retard(xi, yj, zk,ii)
                betalaboxxxint=betalaboxxxint+betalabxxx_retard(xi, yj, zk,ii)
                betalabozxxint=betalabozxxint+betalabzxx_retard(xi, yj, zk,ii)
                betalaboxyyint=betalaboxyyint+betalabxyy_retard(xi, yj, zk,ii)
                betalaboxxyint=betalaboxxyint+betalabxxy_retard(xi, yj, zk,ii)
                betalabozxyint=betalabozxyint+betalabzxy_retard(xi, yj, zk,ii)
            av=av+hx*hy*hz*(betalaboxxxint*np.conjugate(betalaboxxxint))*np.sin(yj)
            ch=ch+hx*hy*hz*(betalabozyyint*np.conjugate(betalabozyyint))*np.sin(yj)
            ah=ah+hx*hy*hz*(betalabozxxint*np.conjugate(betalabozxxint))*np.sin(yj)
            cv=cv+hx*hy*hz*(betalaboxyyint*np.conjugate(betalaboxyyint))*np.sin(yj)
            bv=bv+hx*hy*hz*np.sin(yj)*(4*(betalaboxxyint*np.conjugate(betalaboxxyint))+betalaboxxxint*(np.conjugate(betalaboxyyint))+(betalaboxyyint*(np.conjugate(betalaboxxxint))))
            bv1=bv1+hx*hy*hz*np.sin(yj)*(2*betalaboxxxint*(np.conjugate(betalaboxxyint))+2*betalaboxxyint*(np.conjugate(betalaboxxxint)))
            bv2=bv2+hx*hy*hz*np.sin(yj)*(2*betalaboxyyint*(np.conjugate(betalaboxxyint))+2*betalaboxxyint*(np.conjugate(betalaboxyyint)))
            bh=bh+hx*hy*hz*np.sin(yj)*(4*(betalabozxyint*np.conjugate(betalabozxyint))+betalabozxxint*(np.conjugate(betalabozyyint))+(betalabozyyint*(np.conjugate(betalabozxxint))))
            bh1=bh1+hx*hy*hz*np.sin(yj)*(2*betalabozxxint*(np.conjugate(betalabozxyint))+2*betalabozxyint*(np.conjugate(betalabozxxint)))
            bh2=bh2+hx*hy*hz*np.sin(yj)*(2*betalabozyyint*(np.conjugate(betalabozxyint))+2*betalabozxyint*(np.conjugate(betalabozyyint)))   


print("**********calculation complete ************")           
print(f"av = {av:.5f}")
print(f"bv = {bv:.5f}")
print(f"bv1 = {bv1:.5f}")
print(f"bv2 = {bv2:.5f}")
print(f"cv = {cv:.5f}")
print(f"ah = {ah:.5f}")
print(f"bh = {bh:.5f}")
print(f"bh1 = {bh1:.5f}")
print(f"bh2 = {bh2:.5f}")
print(f"ch =  {ch:.5f}")

av=abs(av)
cv=abs(cv)
ah=abs(ah)
ch=abs(ch)
bv=abs(bv)
bv1=abs(bv1)
bv2=abs(bv2)
bh=abs(bh)
bh1=abs(bh1)
bh2=abs(bh2)

I4v=(av-bv+cv)/((3*av+bv+3*cv))
I2v=(4)*(av-cv)/(3*av+bv+3*cv)
I4h=(ah-bh+ch)/((3*ah+bh+3*ch))
I2h=(4)*(ah-ch)/(3*ah+bh+3*ch)
print(f"I2v = {I2v:.5f}")
print(f"I4v = {I4v:.5f}")
print(f"I2h = {I2h:.5f}")
print(f"I4h = {I4h:.5f}")


with open(uu, "w") as f:        
    f.write(" **************************************************** \n")
    f.write(" ******************program SHS_polar_90************** \n")
    f.write(" **************************************************** \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" Contributors : Pierre-Marie Gassin  &  Gaelle Gassin \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" ********************INPUT************************************ \n")
    f.write("  \n")
    f.write(f" refractive index of the medium: {n}\n")
    f.write(f" wavelength of the incident laser beam (expressed in nm): {longueuronde}\n")
    f.write(f" number of dipoles: {nombre}\n")
    f.write(f" {datacomments} \n")
    f.write(f" hyperpolarisability tensor:   \n")
    f.write(f" {beta} \n")
    f.write("  \n")
    f.write(" ****************OUTPUT*******************************\n")
    f.write("  \n")
    f.write("av %12.6f\n" % av)
    f.write("bv %12.6f\n" % bv)
    f.write("bv1 %12.6f\n" % bv1)
    f.write("bv2 %12.6f\n" % bv2)
    f.write("cv %12.6f\n" % cv)
    f.write("ah %12.6f\n" % ah)
    f.write("bh %12.6f\n" % bh)
    f.write("bh1 %12.6f\n" % bh1)
    f.write("bh2 %12.6f\n" % bh2)
    f.write("ch %12.6f\n" % ch)
    f.write("  \n")
    f.write("*************************************************\n")
    f.write("  \n")
    f.write("I2v %12.6f\n" % I2v)
    f.write("I4v %12.6f\n" % I4v)
    f.write("I2h %12.6f\n" % I2h)   
    f.write("I4h %12.6f\n" % I4h)
    f.write("  \n")
    f.write("*************************************************\n")
    f.write("Have a nice day and enjoy the life!")


gammaa=np.linspace(0,2*np.pi,100)
plt.plot(gammaa*360/(2*np.pi),av*(np.cos(gammaa)**4)+bv*(np.cos(gammaa)**2)*(np.sin(gammaa)**2)+(cv)*(np.sin(gammaa)**4)+(bv1)*(np.cos(gammaa)**3)*(np.sin(gammaa))+(bv2)*(np.cos(gammaa))*(np.sin(gammaa)**3),label='polarV_out')
plt.plot(gammaa*360/(2*np.pi),ah*(np.cos(gammaa)**4)+bh*(np.cos(gammaa)**2)*(np.sin(gammaa)**2)+(ch)*(np.sin(gammaa)**4)+(bh1)*(np.cos(gammaa)**3)*(np.sin(gammaa))+(bh2)*(np.cos(gammaa))*(np.sin(gammaa)**3),label='polarH_out')
plt.legend(loc='upper left')
plt.title("Polarization resolved SHS in the right angle direction (90°)")
plt.ylabel("SHS Intensity")
plt.xlabel("Polarization angle [degree]")
plt.savefig(uuu)
plt.show()

fig = plt.figure()
ax = fig.gca(projection='3d')
ax.quiver(x,y,z,np.sin(phi)*np.sin(theta),np.cos(phi)*np.sin(theta),np.cos(theta),length=(maxtotal)/20, normalize=True)
ax.set_xlim3d(-max(xmax,ymax,zmax),max(xmax,ymax,zmax))
ax.set_ylim3d(-max(xmax,ymax,zmax),max(xmax,ymax,zmax))
ax.set_zlim3d(-max(xmax,ymax,zmax),max(xmax,ymax,zmax))
plt.savefig(vvv)
plt.show()
    


