
#*******************************************************************************
#******************PROGRAM SHS_polar_90_linear******************************************
#*******************************************************************************
#   PySHS - An open source software about Second Harmonic Scattering 
#   developed at Institut Charles Gerhardt Montpellier - ENSCM
#   Dr Pierre-Marie GASSIN
#   Dr Gassin GASSIN
#    (june 2020)  
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#*******************************************************************************

#********************************************************************************
#
# Program for computing polarization resolved SHS Intensity for supramolecular
# structure in solution in the right angle direction according the approach
# developped in Langmuir 2018, 34, 12206−1221 
#  
#********************************************************************************

#*******************************************************************************
# IMPORT MODULES
#*******************************************************************************

import matplotlib.pyplot as plt
import numpy as np
import matplotlib as mpl
import sys
from scipy.integrate import tplquad
from mpl_toolkits.mplot3d import Axes3D


#input and output files
ukv=sys.argv[1]
uv=sys.argv[2]
uu=sys.argv[3]
uuu=sys.argv[3]+".pdf"
vvv=sys.argv[3]+".png"

print("********************************************************")
print("****           PySHS: An open source software       ****")
print("****         about Second Harmonic Scattering       ****") 
print("**** Dr Pierre-Marie GASSIN and Dr Gaelle GASSIN    ****")          
print("****                  ICGM - ENSCM                  ****") 
print('****                   june 2020                    ****')
print("********************************************************")

n=float(input("refractive index of the medium?"))
longueuronde=float(input("wavelength of the incident laser beam (expressed in nm)?"))
nombre=int(input("number of dipoles (ie number of lines in the input file)?"))
grandk=4*np.pi*n/longueuronde
petitk=2*np.pi*n/longueuronde
pas=8

def Rphi(phi):
    aaa=np.array([[np.cos(phi),-np.sin(phi),0],[np.sin(phi),np.cos(phi),0],[0,0,1]])
    return aaa

def Rtheta(theta):
    bb=np.array([[1,0,0],[0,np.cos(theta),-np.sin(theta)],[0,np.sin(theta),np.cos(theta)]])
    return bb

def Rpsi(psi):
    cc=np.array([[np.cos(psi),-np.sin(psi),0],[np.sin(psi),np.cos(psi),0],[0,0,1]])
    return cc


def T(phi,theta,psi):
    zz=np.dot(np.linalg.inv(Rphi(phi)), np.dot(np.linalg.inv(Rtheta(theta)),np.linalg.inv(Rpsi(psi))))
    return zz


beta=np.zeros((3,3,3))
with open(ukv,'r') as f:
    datacomments=f.readlines()[0]

kk=0
data1=[0]*27
value1=[0]*27
beta=np.zeros((3,3,3))
for i in range(3):
    for j in range(3):
        for k in range(3):
            with open(ukv,'r') as f:
                data1[kk]=f.readlines()[kk+1]
                value1[kk] = data1[kk].split()
                beta[i,j,k]=float(value1[kk][1])
                kk=kk+1

print(f" input hyperpolarisability tensor: \n {beta}")

  
nn=np.zeros(nombre)
phi=np.zeros(nombre)
theta=np.zeros(nombre)
psi=np.zeros(nombre)
x=np.zeros(nombre)
y=np.zeros(nombre)
z=np.zeros(nombre)

betameso=np.zeros((3,3,3))
grandgammamesox=0
grandgammamesoy=0
grandgammamesoz=0
data=[0]*nombre
value=[0]*nombre

def betameso_i(phi,theta,psi):
    uuu=np.zeros((3,3,3))
    for i in range(3):
        for j in range(3):
            for k in range(3):
                uuu[i,j,k]=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[0,0,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[0,0,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[0,0,2])+\
                            (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[0,1,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[0,1,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[0,1,2])+\
                            (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[0,2,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[0,2,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[0,2,2])+\
                            (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[1,0,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[1,0,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[1,0,2])+\
                            (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[1,1,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[1,1,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[1,1,2])+\
                            (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[1,2,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[1,2,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[1,2,2])+\
                            (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*beta[2,0,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*beta[2,0,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*beta[2,0,2])+\
                            (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*beta[2,1,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*beta[2,1,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*beta[2,1,2])+\
                            (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*beta[2,2,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*beta[2,2,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*beta[2,2,2])
    return uuu


for i in range(nombre):
    with open(uv,'r') as f:
        data[i]=f.readlines()[i]
        value[i] = data[i].split()
        phi[i]=float(value[i][0])
        theta[i]=float(value[i][1])
        psi[i]=float(value[i][2])
        x[i]=float(value[i][3])
        y[i]=float(value[i][4])
        z[i]=float(value[i][5])
        betameso=betameso+betameso_i(phi[i],theta[i],psi[i])
        grandgammamesox=grandgammamesox+(x[i])*(betameso_i(phi[i],theta[i],psi[i]))
        grandgammamesoy=grandgammamesoy+(y[i])*(betameso_i(phi[i],theta[i],psi[i]))
        grandgammamesoz=grandgammamesoz+(z[i])*(betameso_i(phi[i],theta[i],psi[i]))
        
xmax=np.amax(np.absolute(x))
ymax=np.amax(np.absolute(y))
zmax=np.amax(np.absolute(z))
maxtotal=xmax+ymax+zmax

def grandgammalab_lijk(phi,theta,psi,l,i,j,k):
    uuu=0
    uuu=(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesox[0,0,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesox[0,0,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesox[0,0,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesox[0,1,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesox[0,1,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesox[0,1,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesox[0,2,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesox[0,2,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesox[0,2,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesox[1,0,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesox[1,0,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesox[1,0,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesox[1,1,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesox[1,1,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesox[1,1,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesox[1,2,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesox[1,2,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesox[1,2,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesox[2,0,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesox[2,0,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesox[2,0,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesox[2,1,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesox[2,1,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesox[2,1,2])+\
    (T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesox[2,2,0])+T(phi,theta,psi)[l,0]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesox[2,2,1])+(T(phi,theta,psi)[l,0]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesox[2,2,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesoy[0,0,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesoy[0,0,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesoy[0,0,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesoy[0,1,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesoy[0,1,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesoy[0,1,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesoy[0,2,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesoy[0,2,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesoy[0,2,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesoy[1,0,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesoy[1,0,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesoy[1,0,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesoy[1,1,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesoy[1,1,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesoy[1,1,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesoy[1,2,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesoy[1,2,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesoy[1,2,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesoy[2,0,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesoy[2,0,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesoy[2,0,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesoy[2,1,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesoy[2,1,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesoy[2,1,2])+\
    (T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesoy[2,2,0])+T(phi,theta,psi)[l,1]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesoy[2,2,1])+(T(phi,theta,psi)[l,1]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesoy[2,2,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesoz[0,0,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesoz[0,0,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesoz[0,0,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesoz[0,1,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesoz[0,1,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesoz[0,1,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesoz[0,2,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesoz[0,2,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesoz[0,2,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesoz[1,0,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesoz[1,0,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesoz[1,0,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesoz[1,1,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesoz[1,1,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesoz[1,1,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesoz[1,2,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesoz[1,2,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesoz[1,2,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*grandgammamesoz[2,0,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*grandgammamesoz[2,0,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*grandgammamesoz[2,0,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*grandgammamesoz[2,1,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*grandgammamesoz[2,1,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*grandgammamesoz[2,1,2])+\
    (T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*grandgammamesoz[2,2,0])+T(phi,theta,psi)[l,2]*(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*grandgammamesoz[2,2,1])+(T(phi,theta,psi)[l,2]*T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*grandgammamesoz[2,2,2])
    return uuu

def betalabijk(phi,theta,psi,i,j,k):
    uuu=0
    uuu=(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betameso[0,0,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betameso[0,0,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betameso[0,0,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betameso[0,1,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betameso[0,1,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betameso[0,1,2])+\
    (T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betameso[0,2,0])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betameso[0,2,1])+(T(phi,theta,psi)[i,0]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betameso[0,2,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betameso[1,0,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betameso[1,0,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betameso[1,0,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betameso[1,1,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betameso[1,1,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betameso[1,1,2])+\
    (T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betameso[1,2,0])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betameso[1,2,1])+(T(phi,theta,psi)[i,1]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betameso[1,2,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,0]*betameso[2,0,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,1]*betameso[2,0,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,0]*T(phi,theta,psi)[k,2]*betameso[2,0,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,0]*betameso[2,1,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,1]*betameso[2,1,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,1]*T(phi,theta,psi)[k,2]*betameso[2,1,2])+\
    (T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,0]*betameso[2,2,0])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,1]*betameso[2,2,1])+(T(phi,theta,psi)[i,2]*T(phi,theta,psi)[j,2]*T(phi,theta,psi)[k,2]*betameso[2,2,2])
    return uuu

def grandgammalabxxx(phi,theta,psi):
    uuu=0
    uuu=(4*(np.pi)*n/longueuronde)*(grandgammalab_lijk(phi,theta,psi,2,0,0,0)+grandgammalab_lijk(phi,theta,psi,1,0,0,0))
    return uuu

def grandgammalabzxx(phi,theta,psi):
    uuu=0
    uuu=(4*(np.pi)*n/longueuronde)*(grandgammalab_lijk(phi,theta,psi,2,2,0,0)+grandgammalab_lijk(phi,theta,psi,1,2,0,0))
    return uuu

def grandgammalabxyy(phi,theta,psi):
    uuu=0
    uuu=(4*(np.pi)*n/longueuronde)*(grandgammalab_lijk(phi,theta,psi,2,0,1,1)+grandgammalab_lijk(phi,theta,psi,1,0,1,1))
    return uuu

def grandgammalabzyy(phi,theta,psi):
    uuu=0
    uuu=(4*(np.pi)*n/longueuronde)*(grandgammalab_lijk(phi,theta,psi,2,2,1,1)+grandgammalab_lijk(phi,theta,psi,1,2,1,1))
    return uuu

def grandgammalabxxy(phi,theta,psi):
    uuu=0
    uuu=2*(4*(np.pi)*n/longueuronde)*(grandgammalab_lijk(phi,theta,psi,2,0,0,1)+grandgammalab_lijk(phi,theta,psi,1,0,0,1))
    return uuu

def grandgammalabzxy(phi,theta,psi):
    uuu=0
    uuu=2*(4*(np.pi)*n/longueuronde)*(grandgammalab_lijk(phi,theta,psi,2,2,0,1)+grandgammalab_lijk(phi,theta,psi,1,2,0,1))
    return uuu

def betalabxxx(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,0,0,0)
    return uuu

def betalabxyy(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,0,1,1)
    return uuu

def betalabzxx(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,2,0,0)
    return uuu
    
def betalabzyy(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,2,1,1)
    return uuu

def betalabxxy(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,0,0,1)
    return uuu
    
def betalabzxy(phi,theta,psi):
    uuu=0
    uuu=betalabijk(phi,theta,psi,2,0,1)
    return uuu


av=0
bv=0
cv=0
ah=0
bh=0
ch=0
avretard=0
bvretard=0
cvretard=0
ahretard=0
bhretard=0
chretard=0
avt=0
bvt=0
cvt=0
aht=0
bht=0
cht=0
print("********** Begin calculation **************")
for i in range(2*pas):
    fgf=float(i/(2*pas))*100
    print(f"calculation in progress: {fgf:.0f} %")
    for j in range(pas):
        for k in range(2*pas):
            av=av+((np.pi/pas)**3)*(((betalabxxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            bv=bv+((np.pi/pas)**3)*(4*(((betalabxxy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))**2)+2*(betalabxxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))*(betalabxyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))*(np.sin(j*np.pi/pas))
            cv=cv+((np.pi/pas)**3)*(((betalabxyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            ah=ah+((np.pi/pas)**3)*(((betalabzxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            ch=ch+((np.pi/pas)**3)*(((betalabzyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            bh=bh+((np.pi/pas)**3)*(4*(((betalabzxy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))**2)+2*(betalabzxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))*(betalabzyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))*(np.sin(j*np.pi/pas))
            avretard=avretard+((np.pi/pas)**3)*(((grandgammalabxxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            cvretard=cvretard+((np.pi/pas)**3)*(((grandgammalabxyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            ahretard=ahretard+((np.pi/pas)**3)*(((grandgammalabzxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            chretard=chretard+((np.pi/pas)**3)*(((grandgammalabzyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))**2)*(np.sin(j*np.pi/pas)))
            bvretard=bvretard+((np.pi/pas)**3)*(1*(((grandgammalabxxy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))**2)+\
                                  2*(grandgammalabxxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas)))*(grandgammalabxyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))*(np.sin(j*np.pi/pas))
            bhretard=bhretard+((np.pi/pas)**3)*(1*(((grandgammalabzxy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))**2)+\
                                  2*(grandgammalabzxx(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))\
                                     )*(grandgammalabzyy(i*(np.pi/pas),j*(np.pi/pas),k*(np.pi/pas))))*(np.sin(j*np.pi/pas))

                
avt=av+avretard
bvt=bv+bvretard
cvt=cv+cvretard
aht=ah+ahretard
bht=bh+bhretard
cht=ch+chretard

print("**********calculation complete ************")           
print(f"av = {avt:.5f}")
print(f"bv = {bvt:.5f}")
print(f"cv = {cvt:.5f}")
print(f"ah = {aht:.5f}")
print(f"bh = {bht:.5f}")
print(f"ch =  {cht:.5f}")

I4v=(avt-bvt+cvt)/(3*avt+bvt+3*cvt)
I2v=(4)*(avt-cvt)/(3*avt+bvt+3*cvt)
I4h=(aht-bht+cht)/(3*aht+bht+3*cht)
I2h=(4)*(aht-cht)/(3*aht+bht+3*cht)

print(f"I2v = {I2v:.5f}")
print(f"I4v = {I4v:.5f}")
print(f"I2h = {I2h:.5f}")
print(f"I4h = {I4h:.5f}")

with open(uu, "w") as f:        
    f.write(" ********************************************************* \n")
    f.write(" ************program SHS_polar_90_linear****************** \n")
    f.write(" ********************************************************* \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" Contributors : Pierre-Marie Gassin  &  Gaelle Gassin \n")
    f.write("  \n")
    f.write("  \n")
    f.write(" ********************INPUT***************************** \n")
    f.write("  \n")
    f.write(f" refractive index of the medium: {n}\n")
    f.write(f" wavelength of the incident laser beam (expressed in nm): {longueuronde}\n")
    f.write(f" number of dipoles: {nombre}\n")
    f.write(f" {datacomments} \n")
    f.write(f" hyperpolarisability tensor:   \n")
    f.write(f" {beta} \n")
    f.write("  \n")
    f.write(" ****************OUTPUT*******************************\n")
    f.write("  \n")
    f.write("avtotal %12.6f\n" % avt)
    f.write("bvtotal %12.6f\n" % bvt)
    f.write("cvtotal %12.6f\n" % cvt)
    f.write("ahtotal %12.6f\n" % aht)
    f.write("bhtotal %12.6f\n" % bht)
    f.write("chtotal %12.6f\n" % cht)
    f.write("  \n")
    f.write("av %12.6f\n" % av)
    f.write("avretard %12.6f\n" % avretard)
    f.write("bv %12.6f\n" % bv)
    f.write("bvretard %12.6f\n" % bvretard)
    f.write("cv %12.6f\n" % cv)
    f.write("cvretard %12.6f\n" % cvretard)
    f.write("ah %12.6f\n" % ah)
    f.write("ahretard %12.6f\n" % ahretard)
    f.write("bh %12.6f\n" % bh)
    f.write("bhretard %12.6f\n" % bhretard)
    f.write("ch %12.6f\n" % ch)
    f.write("chretard %12.6f\n" % chretard)
    f.write("*************************************************\n")
    f.write("  \n")
    f.write("I2v %12.6f\n" % I2v)
    f.write("I4v %12.6f\n" % I4v)
    f.write("I2h %12.6f\n" % I2h)   
    f.write("I4h %12.6f\n" % I4h)
    f.write("  \n")
    f.write("*************************************************\n")
    f.write("Have a nice day and enjoy the life!")

gammaa=np.linspace(0,2*np.pi,100)
plt.plot(gammaa*360/(2*np.pi),avt*(np.cos(gammaa)**4)+bvt*(np.cos(gammaa)**2)*(np.sin(gammaa)**2)+(cvt)*(np.sin(gammaa)**4),label='V_out')#+(bv1)*(np.cos(gammaa)**3)*(np.sin(gammaa))+(bv2)*(np.cos(gammaa))*(np.sin(gammaa)**3),legend='polarV_out')
plt.plot(gammaa*360/(2*np.pi),aht*(np.cos(gammaa)**4)+bht*(np.cos(gammaa)**2)*(np.sin(gammaa)**2)+(cht)*(np.sin(gammaa)**4),label='H_out')#+(bh1)*(np.cos(gammaa)**3)*(np.sin(gammaa))+(bh2)*(np.cos(gammaa))*(np.sin(gammaa)**3),legend='polarH_out')
plt.legend(loc='upper left')
plt.title("Polarization resolved SHS in the right angle direction (90°)")
plt.ylabel("SHS Intensity")
plt.xlabel("Polarization angle [degree]")
plt.savefig(uuu)
plt.show()

fig = plt.figure()
ax = fig.gca(projection='3d')
ax.quiver(x,y,z,np.sin(phi)*np.sin(theta),np.cos(phi)*np.sin(theta),np.cos(theta),length=(maxtotal)/20, normalize=True)
plt.savefig(vvv)
plt.show()
